﻿Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Threading
Imports Sock
Imports Sock.F
Imports SockSample.Functions


Public Class Client



#Region "クライアントソケット"
    Private WithEvents SendSock As New SendSockClass   'クライアントソケット


    Delegate Sub SendSockConnectDelegate()
    Private Sub SendSockConnect()
        If InvokeRequired Then
            ' 別スレッドから呼び出された場合
            Invoke(New SendSockConnectDelegate(AddressOf SendSockConnect))
            Return
        End If
        '再接続を行う
        With SendSock
            .RemoteIP = RemoteIP.Text
            .RemotePort = RemotePort.Text
            .ConnectTimeOut = 1000
            .ConnectRetry = 2
            .ConnectRetryWait = 3000

            .SockConnect()
        End With

    End Sub



    ''' <summary>
    ''' ソケット送信処理
    ''' </summary>
    ''' <remarks>接続中のクライアントソケットに対して送信を行う</remarks>
    Private Sub SendSockSend()
        'If InvokeRequired Then
        '    ' 別スレッドから呼び出された場合
        '    Invoke(New SendSockSendDelegate(AddressOf SendSockSend))
        '    Return
        'End If
        With SendSock
            Dim sendParm As New SendSockClass.SendParmClass
            With sendParm
                .SendKEY = "TIME" & Format(Now, "HHmmss.fff") '送信コマンドを識別する値
                .sndCmd = fGetCmd()


                .SendTimeOut = 1000
                .SendRetry = 3
                .SendRetryWait = 2000

                .IsRecv = True '送信完了後に結果を受信する
                .RecvTimeOut = 5000 '結果受信を待つ時間
            End With

            .SockSend(sendParm)
        End With
    End Sub

    ''' <summary>
    ''' 接続完了
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnConnectEnd(ByVal sender As Object, ByVal e As SockConnectEndEventArgs) Handles SendSock.OnConnectEnd
        Call DispMsg("[OnConnectEnd]" & e.LocalIP & ":" & e.LocalPort & " -> " & e.RemoteIP & ":" & e.RemotePort & " 接続")

    End Sub

    ''' <summary>
    ''' 接続リトライ発生
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnConnectRetry(ByVal sender As Object, ByVal e As SockConnectRetryEventArgs) Handles SendSock.OnConnectRetry
        Call DispMsg("[OnConnectRetry]" & e.RemoteIP & ":" & e.RemotePort & " 接続リトライ " & e.RetryDo & "回目")
    End Sub

    ''' <summary>
    ''' クライアントソケットでエラー発生
    ''' </summary>
    ''' <param name="sender">sender</param>
    ''' <param name="e">e</param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnError(ByVal sender As Object, ByVal e As SockErrorEventArgs) Handles SendSock.OnError
        Call DispMsg("[OnError]クライアントソケットエラー検知:" & e.ErrorType & ":" & e.Msg)
        Select Case e.ErrorType
            Case SockErrors.ConnectErr
                Call DispMsg("⇒[ConnectErr]接続エラー発生:" & e.Msg)
            Case SockErrors.ConnectRetryOver
                Call DispMsg("⇒[ConnectRetryOver]接続リトライオーバー。再接続を行います:" & e.Msg)
                '再接続を自動で行う場合
                'IsSendSockInit = True
            Case SockErrors.SendErr
                Call DispMsg("⇒[SendErr]送信エラー:" & e.Msg)
            Case SockErrors.SendRetryOver
                Call DispMsg("⇒[SendRetryOver]リトライオーバーしました:" & e.Msg)
            Case SockErrors.RecvErr
                Call DispMsg("⇒[RecvErr]結果待ちタイムアウト:" & e.ErrorType & ":" & e.Msg)
        End Select
    End Sub

    ''' <summary>
    ''' クライアントソケットからのメッセージ
    ''' </summary>
    ''' <param name="sender">sender</param>
    ''' <param name="e">e</param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnMessage(ByVal sender As Object, ByVal e As SockMessageEventArgs) Handles SendSock.OnMessage
        Call DispMsg("[OnMessage]" & e.Msg)

    End Sub

    ''' <summary>
    ''' 受信完了
    ''' </summary>
    ''' <param name="sender">sender</param>
    ''' <param name="e">e</param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnRecvEnd(ByVal sender As Object, ByVal e As SockReceiveEndEventArgs) Handles SendSock.OnRecvEnd
        Call DispMsg("[OnRecvEnd]" & e.RemoteIP & ":" & e.RemotePort & " -> " & e.LocalIP & e.LocalPort & " 送信Key=" & e.SendKey & " " & Byte2String(HexChar2Byte(e.RecvChar)) & "(" & e.RecvChar & ")")
    End Sub

    ''' <summary>
    ''' 送信リトライ発生
    ''' </summary>
    ''' <param name="sender">sender</param>
    ''' <param name="e">e</param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnSendRetry(ByVal sender As Object, ByVal e As SockSendRetryEventArgs) Handles SendSock.OnSendRetry
        Call DispMsg("[OnSendRetry]" & e.RemoteIP & ":" & e.RemotePort & " へ送信リトライ " & e.RetryDo & "回目 送信Key=" & e.SendKey)
    End Sub

    ''' <summary>
    ''' 送信完了
    ''' </summary>
    ''' <param name="sender">sender</param>
    ''' <param name="e">e</param>
    ''' <remarks></remarks>
    Private Sub SendSock_OnSendEnd(ByVal sender As Object, ByVal e As SockSendEndEventArgs) Handles SendSock.OnSendEnd
        Call DispMsg("[OnSendEnd]" & e.RemoteIP & ":" & e.RemotePort & " へ送信->" & e.SendChar & " 送信Key=" & e.SendKey)
    End Sub


    Private Sub SendSockConnectBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SendSockConnectBtn.Click
        'Call SendSockStart()
        Call SendSockConnect()
    End Sub

    Private Sub SendSockSendBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SendSockSendBtn.Click
        Call SendSockSend()
    End Sub

    Private Sub SendSockDisConnectBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SendSockDisConnectBtn.Click
        Call SendSock.CloseSock()
    End Sub

    Private Sub StatusCheck_Click(sender As System.Object, e As System.EventArgs) Handles StatusCheck.Click
        If SendSock.IsAlive Then
            Call DispMsg("接続中")
        Else
            Call DispMsg("切断")
        End If
    End Sub


#End Region



#Region "メッセージ表示関連"
    Delegate Sub DispMsgDelegate(ByVal msg As String)

    'メッセージ表示
    Public Sub DispMsg(ByVal msg As String)
        If InvokeRequired Then
            ' 別スレッドから呼び出された場合
            Invoke(New DispMsgDelegate(AddressOf DispMsg), msg)
            Return
        End If

        Dim lastMsg As String
        With lstMsg
            If .Items.Count = 0 Then
                lastMsg = ""
            Else
                lastMsg = .Items(.Items.Count - 1).ToString
            End If
        End With
        Call ListBoxAdd(lstMsg, Format(Now, "MM/dd HH:mm:ss.ff ") & msg, 100)

    End Sub

    'リストボックスに１行追加表示する
    Private Sub ListBoxAdd(ByVal lb As ListBox, ByVal msg As String, ByVal maxLine As Long)
        With lb

            .Items.Add(msg)
            If maxLine <> 0 Then '0の時は削除しない
                If .Items.Count > maxLine Then
                    .Items.RemoveAt(0)
                End If
            End If
            .SetSelected(.Items.Count - 1, True)
            .Refresh()
        End With

    End Sub

#End Region



    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.RemoteIP.Text = getip()

    End Sub

    Private Function fGetCmd() As Byte()
        Dim strCmd As String = String.Empty
        strCmd &= Me.SendText.Text
        Return String2Byte(strCmd)
    End Function




    Private Sub AutoSendTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AutoSendTimer.Tick
        If DateTime.Now.Second Mod 10 = 0 Then
            Call SendSockSend()
        End If
    End Sub


End Class
